@@ -6,24 +6,26 @@ import tempfile
 import json
 import calendar
 import datetime
-import traceback
+import sys
 import argparse
 from seams.severity import Severity
+from traceback import format_tb
 class Pipeline(ABC):
     
 
-    def __init__(self, EMAIL, PASSWORD, URL=None):
+    def __init__(self, SECRET, APP_ID, TENANT_ID, URL=None):
         '''
         Create an instance of the Pipeline interface
         '''
         self.result_files = []
         self.logs = []
         self.user_logs = []
+        self.secret = SECRET
         if URL:
             self.seams = Seams(URL)
         else:
             self.seams = Seams()
-        self.seams.connect(EMAIL, PASSWORD)
+        self.seams.connect(SECRET, APP_ID, TENANT_ID)
 
     
     @abstractmethod
@@ -44,35 +46,33 @@ class Pipeline(ABC):
         try:
             #getting parameters and setting them to appropriate class variables
             parameters = self.__get_parameters()
-            self.vertex_id = parameters['vertex_id']
-            self.tenant_id = parameters['tenant_id']
+            self.vertex_id = parameters["vertex_id"]
+            self.tenant_id = parameters["tenant_id"]
             #setting pipeline status to IN PROGRESS
             self.update_pipeline_status_in_progress() 
             #getting the run parameters of pipeline
-            data = json.loads(self.get_pipeline_run()['runParameters'])
+            data = json.loads(self.seams.get_vertex_by_id(self.tenant_id, self.vertex_id)["runParameters"])
 
             #downloading files
             self.log(Severity.INFO, "downloading files...")
-            data['files'] = self.download_input_files()
-
+            data["files"] = self.download_input_files()
+            # if len(data['files']) < 5:
+            #     raise FileNotFoundError("Not enough files provided for this pipeline")
             #running abstract run method
             self.run(data)
             #setting pipeline to DONE when finished
             return self.update_pipeline_status_done()
         except Exception as e:
+            print("exception: ", e)
             #if pipeline fails setting pipeline to ERROR and printing out the stack trace
+            etype, value, tb = sys.exc_info()
+            #formats the exception type into a string and grabs only the class type of the error
+            exception_type = str(type(value)).replace("'", "").split(" ")[1][:-1]
+            if exception_type == "KeyError":
+                self.log(Severity.ERROR, exception_type, ": Input parameter ", str(value).replace("'", ""), " does not match any of the expected arguments for this pipeline", for_user=True)
+            self.log(Severity.ERROR, format_tb(tb))
+            self.log(Severity.ERROR, exception_type, ": ", str(value).replace("'", ""), for_user=True)
             self.update_pipeline_status_error()
-            self.log(Severity.ERROR, traceback.print_exc())
-
-
-    def get_pipeline_run(self):
-        '''
-        get the pipeline vertex
-
-        :returns:
-            JSON object representing the pipeline vertex
-        '''
-        return self.seams.get_vertex_by_id(self.tenant_id, self.vertex_id)
 
 
     def update_pipeline_status_in_progress(self):
@@ -88,7 +88,6 @@ class Pipeline(ABC):
         return self.seams.update_vertex(self.tenant_id, self.vertex_id, "PipelineRun", attributes)
     
 
-
     def update_pipeline_status_done(self):
         '''
         sets the pipeline status to DONE
@@ -113,11 +112,12 @@ class Pipeline(ABC):
         :returns:
             JSON object representing the status of the completed pipeline
         '''
+        upload_log_files = self.__build_log_upload_files()
         attributes = {
             "status": "ERROR",
             "runResults": self.result_files,
-            "logs": self.logs,
-            "userLogs": self.user_logs
+            "logs": upload_log_files[0],
+            "userLogs": upload_log_files[1]
         }
         return self.seams.update_vertex(self.tenant_id, self.vertex_id, "PipelineRun", attributes)
 
@@ -130,28 +130,28 @@ class Pipeline(ABC):
             list of file paths for files downloaded for the pipeline
         '''
         #gets the run parameters of the pipeline
-        data = json.loads(self.seams.get_vertex_by_id(self.tenant_id, self.vertex_id)['runParameters'])
+        data = json.loads(self.seams.get_vertex_by_id(self.tenant_id, self.vertex_id)["runParameters"])
         files = []
         #downloads files in pipeline, adds them to a temp file, adds file path to a list
-        for item in data['Files']:
-            download = self.seams.download_files(self.tenant_id, item['vertexId'])
+        for item in data["Files"]:
+            download = self.seams.download_files(self.tenant_id, item["vertexId"])
             for item in download:
                 temp_file_full_path = os.path.join(tempfile.gettempdir(), item)
                 files.append(temp_file_full_path)
-                f = open(temp_file_full_path, 'w', encoding="utf-8")
+                f = open(temp_file_full_path, "w", encoding="utf-8")
                 f.write(download[item])
                 f.close()
         return files
     
 
-    def upload_files(self, caption, files):
+    def upload_files(self, caption, files, file_type):
         '''
         uploads any files 
 
         :returns:
             JSON object representing the status of the upload
         '''
-        return self.seams.upload_files(self.tenant_id, caption, files)
+        return self.seams.upload_files(self.tenant_id, caption, files, file_type=file_type)
 
 
     def get_result_files(self):
@@ -175,6 +175,13 @@ class Pipeline(ABC):
             "name": name
         }
         self.result_files.append(new_file)
+    
+
+    def get_tenant_id(self):
+        '''
+        Returns tenant id
+        '''
+        return self.tenant_id
 
 
     def log(self, log_severity, *args, for_user=False, print_out=True):
@@ -201,8 +208,9 @@ class Pipeline(ABC):
                     temp = ""
                     if isinstance(arg[0], dict):
                         for item in arg:
-                            temp = temp + json.dumps(item)
-                        result = result + "".join(temp)
+                            temp = temp + json.dumps(item) + ", "
+                        temp = temp.rstrip(temp[-2:])
+                        result = result + "[" + "".join(temp) + "]"
                     else:
                         result = result + " ".join(arg)
                 elif isinstance(arg, dict):
@@ -215,30 +223,43 @@ class Pipeline(ABC):
                 "date": utc_time,
                 "message": result
             }
-            str_log = "{}   {}   {}".format(log['severity'], date, log['message'])
+            #builds readable string and prints it out
+            str_log = "{}   {}   {}".format(log["severity"], date, log["message"])
             if print_out:
                 print(str_log)
+            #appends to logs and user_logs if requested
             self.logs.append(log)
             if for_user:
                 self.user_logs.append(log)
 
 
     def __build_log_upload_files(self):
-            log_temp_file_full_path = os.path.join(tempfile.gettempdir(), "logs.json")
-            log_file = open(log_temp_file_full_path, 'w', encoding="utf-8")
-            log_file.write(str(self.logs))
-            log_file.close()
-            log_file_upload = self.upload_files("logs for {}".format(self.vertex_id), log_temp_file_full_path)
-            self.seams.attach_edges(self.tenant_id, self.vertex_id, "Logs", [log_file_upload[0]['id']])
+        '''
+        private method to build and upload log files
 
-            user_log_temp_file_full_path = os.path.join(tempfile.gettempdir(), "user_logs.json")
-            user_log_file = open(user_log_temp_file_full_path, 'w', encoding="utf-8")
-            user_log_file.write(str(self.user_logs))
-            user_log_file.close()
-            user_log_file_upload = self.upload_files("user logs for {}".format(self.vertex_id), user_log_temp_file_full_path)
-            self.seams.attach_edges(self.tenant_id, self.vertex_id, "UserLogs", [user_log_file_upload[0]['id']])
+        :returns:
+            (vertex_id of log file, vertex_id of user_log file)
+        '''
+        #creates new tempfile for logs
+        log_temp_file_full_path = os.path.join(tempfile.gettempdir(), "logs.json")
+        #opens/writes/closes new temp log file
+        log_file = open(log_temp_file_full_path, "w", encoding="utf-8")
+        log_file.write(str(self.logs))
+        log_file.close()
+        #uploads log file and attaches it as an edge to the pipeline run
+        log_file_upload = self.upload_files("logs for {}".format(self.vertex_id), log_temp_file_full_path, file_type="File")
+        self.seams.attach_edges(self.tenant_id, self.vertex_id, "Logs", [log_file_upload[0]["id"]])
+        #creates new tempfile for user_logs
+        user_log_temp_file_full_path = os.path.join(tempfile.gettempdir(), "user_logs.json")
+        #opens/writes/closes new temp user_log file
+        user_log_file = open(user_log_temp_file_full_path, "w", encoding="utf-8")
+        user_log_file.write(str(self.user_logs))
+        user_log_file.close()
+        #uploads user_log file and attaches it as an edge to the pipeline run
+        user_log_file_upload = self.upload_files("user logs for {}".format(self.vertex_id), user_log_temp_file_full_path, file_type="File")
+        self.seams.attach_edges(self.tenant_id, self.vertex_id, "UserLogs", [user_log_file_upload[0]["id"]])
 
-            return (log_file_upload[0]['id'], user_log_file_upload[0]['id'])
+        return (log_file_upload[0]["id"], user_log_file_upload[0]["id"])
 
 
     def __get_parameters(self):
@@ -259,5 +280,5 @@ class Pipeline(ABC):
             parser.error(
                 "Tenant Id and Vertex Id required - do -h for help")
 
-        parameters = {'tenant_id': args.tenant_id, 'vertex_id': args.vertex_id}
+        parameters = {"tenant_id": args.tenant_id, "vertex_id": args.vertex_id}
         return parameters
