@@ -1,9 +1,14 @@
 #!/usr/bin/env python
 from requests import Request, Session
+from itertools import islice
 import sys
 sys.path.append( '.\\seams\\' )
+sys.path.append( '.\\' )
 from seams.exceptions import SeamsException, SeamsAPIException
 import json
+import msal
+import os
+
 
 class Seams(object):
     '''
@@ -21,28 +26,37 @@ class Seams(object):
 
 
     def connect(self, 
-                email, 
-                password):
+                secret,
+                app_id,
+                tenant_id):
         '''
         Connect to the Seams API 
-        :param email:
-            The login email for the Seams API  **REQUIRED**
-        :param password:
-            The login password for the Seams API  **REQUIRED**
+        :param secret:
+            A secret given by RJLG staff to access the Seams SDK  **REQUIRED**
+        :param app_id:
+            An application ID given by RJLG staff to access the Seams SDK  **REQUIRED**
+        :param tenant_id:
+            A tenant ID given by RJLG staff to access the Seams SDK  **REQUIRED**
         :returns:
-            A vertex
+            None
         '''
+        self.secret = secret
+        self.app_id = app_id
+        self.tenant_id = tenant_id
+        app = msal.ConfidentialClientApplication(
+                app_id, 
+                authority="https://login.microsoftonline.com/{}".format(tenant_id),
+                client_credential=secret)
+        result = None
+        result = app.acquire_token_silent(scopes=["https://partiddevb2c.onmicrosoft.com/{}/.default".format(app_id)], account=None)
 
-        body = {'email': email, 'password': password}
-        session = Session()
-        request = Request('POST', '{}/auth/sign-in'.format(self.URL), data=body)
-        prepared = session.prepare_request(request)
-        response = session.send(prepared)
-        if response.text == 'Authentication failed: User or password is incorrect':
-            raise SeamsAPIException(response.text)
+        if not result:
+            result = app.acquire_token_for_client(scopes=["https://partiddevb2c.onmicrosoft.com/{}/.default".format(app_id)])
+        if 'error' in result:
+            print("Error connecting with given credentials")
+            raise SeamsAPIException(result['error_description'])
         else:
-            self.bearer = response.text
-            self.email = email
+            self.bearer = result['access_token']
             self.connected = True
 
 
@@ -51,7 +65,7 @@ class Seams(object):
         Disconnect from the Seams API 
         '''
         self.bearer = None
-        self.email = None
+        self.secret = None
         self.connected = False
 
 
@@ -64,7 +78,7 @@ class Seams(object):
         '''
         
         try:
-            response = self.__http_request(self.bearer, 'GET', '{}/auth/me'.format(self.URL))
+            response = self.__http_request('GET', '{}/auth/me'.format(self.URL))
         except:
             raise SeamsAPIException(response)
         return response.json()
@@ -75,15 +89,17 @@ class Seams(object):
         Gives info on the connection information of the current Seams object
 
         :returns:
-            The bearer token, URL, email and password
+            The bearer token, URL, connected status, and secret
         '''
 
         try:
             response = {
                 'bearer': self.bearer,
                 'url': self.URL,
-                'email': self.email,
-                'connected': self.connected
+                'connected': self.connected,
+                'secret': self.secret,
+                'app_id': self.app_id,
+                'tenant_id': self.tenant_id
             }
             return response
         except:
@@ -105,7 +121,7 @@ class Seams(object):
         '''
 
         try:
-            response = self.__http_request(self.bearer, 'GET', '{}/tenant/{}/vertex/{}'.format(self.URL, tenant_id, vertex_id))
+            response = self.__http_request('GET', '{}/tenant/{}/vertex/{}'.format(self.URL, tenant_id, vertex_id))
             return response.json()
         except:
             raise SeamsAPIException(response.text)
@@ -126,7 +142,7 @@ class Seams(object):
         '''
 
         try:
-            response = self.__http_request(self.bearer, 'GET', '{}/tenant/{}/vertices/{}'.format(self.URL, tenant_id, vertex_label))
+            response = self.__http_request('GET', '{}/tenant/{}/vertices/{}'.format(self.URL, tenant_id, vertex_label))
             return response.json()
         except:
             raise SeamsAPIException(response)
@@ -153,7 +169,9 @@ class Seams(object):
         '''
         body = self.__properties_formatter(vertex_label, attributes)
         try:
-            response = self.__http_request(self.bearer, 'PUT', '{}/tenant/{}/vertex/update/{}'.format(self.URL, tenant_id, vertex_id), content='application/json', data=body)
+            for attributeChunk in self.__chunk_attributes(attributes, 10):
+                body = self.__properties_formatter(vertex_label, attributeChunk)
+                response = self.__http_request('PUT', '{}/tenant/{}/vertex/update/{}'.format(self.URL, tenant_id, vertex_id), content='application/json', data=body)
             return response.json()
         except:
             raise SeamsAPIException(response.text)
@@ -161,7 +179,8 @@ class Seams(object):
 
     def create_vertex(self, 
                       tenant_id, 
-                      vertex_label, 
+                      vertex_label,
+                      vertex_name, 
                       attributes=None):
         '''
         Create a vertex
@@ -169,7 +188,9 @@ class Seams(object):
         :param tenant_id:
             The id of the tenant the creation is on  **REQUIRED**
         :param vertex_label:
-            The label of the vertex the user is getting  **REQUIRED**
+            The label of the vertex the user is creating  **REQUIRED**
+        :param vertex_name:
+            The name of the vertex the user is creating **REQUIRED**
         :param attributes:
             A dictionary of key/value pairs that will represent the data fields of the vertex  **REQUIRED**
         :returns:
@@ -177,14 +198,56 @@ class Seams(object):
         '''
         body = {}
         if attributes:
-            body = self.__properties_formatter(vertex_label, attributes)
+            body = self.__properties_formatter(vertex_label, attributes, vertex_name=vertex_name)
         try:
-            response = self.__http_request(self.bearer, 'POST', '{}/tenant/{}/vertex/create'.format(self.URL, tenant_id), content='application/json', data=body)
+            for attributeChunk in self.__chunk_attributes(attributes, 10):
+                body = self.__properties_formatter(vertex_label, attributeChunk)
+                response = self.__http_request('POST', '{}/tenant/{}/vertex/create'.format(self.URL, tenant_id), content='application/json', data=body)
             return response.json()
         except:
             raise SeamsAPIException(response.text)
 
 
+    def upsert_vertex(self, 
+                      tenant_id, 
+                      vertex_label,
+                      vertex_name, 
+                      attributes=None):
+        '''
+        Create a vertex
+
+        :param tenant_id:
+            The id of the tenant the creation is on  **REQUIRED**
+        :param vertex_label:
+            The label of the vertex the user is creating  **REQUIRED**
+        :param vertex_name:
+            The name of the vertex the user is creating **REQUIRED**
+        :param attributes:
+            A dictionary of key/value pairs that will represent the data fields of the vertex  **REQUIRED**
+        :returns:
+            A JSON formatted object representing the new vertex
+        '''
+        body = {}
+        if attributes:
+            body = self.__properties_formatter(vertex_label, {"name":vertex_name}, vertex_name=vertex_name)
+        try:
+            response = self.__http_request('GET', '{}/tenant/{}/vertices/{}'.format(self.URL, tenant_id, vertex_label))
+            
+            if(not response.json()["count"]):
+                response = self.__http_request('POST', '{}/tenant/{}/vertex/create'.format(self.URL, tenant_id), content='application/json', data=body)
+                vertex_id = response.json()["id"]
+            else:
+                vertex_id = response.json()["vertices"][0]["id"]
+            
+            for attributeChunk in self.__chunk_attributes(attributes, 10):
+                body = self.__properties_formatter(vertex_label, attributeChunk)
+                updateResponse = self.__http_request('PUT', '{}/tenant/{}/vertex/update/{}'.format(self.URL, tenant_id, vertex_id), content='application/json', data=body)
+            
+            return updateResponse.json()
+        except:
+            raise SeamsAPIException(response.text)
+
+
     def delete_vertex(self, 
                       tenant_id, 
                       vertex_id):
@@ -200,7 +263,7 @@ class Seams(object):
         '''
 
         try:
-            response = self.__http_request(self.bearer, 'DELETE', '{}/tenant/{}/vertex/delete/{}'.format(self.URL, tenant_id, vertex_id))
+            response = self.__http_request('DELETE', '{}/tenant/{}/vertex/delete/{}'.format(self.URL, tenant_id, vertex_id))
             return response.text
         except:
             raise SeamsAPIException(response.text)
@@ -227,7 +290,7 @@ class Seams(object):
         '''
 
         try:
-            response = self.__http_request(self.bearer, 'GET', '{}/tenant/{}/edgeVertices/{}/{}/{}'.format(self.URL, tenant_id, vertex_id, edge_label, direction))
+            response = self.__http_request('GET', '{}/tenant/{}/edgeVertices/{}/{}/{}'.format(self.URL, tenant_id, vertex_id, edge_label, direction))
             return response.json()
         except:
             raise SeamsAPIException(response.text)
@@ -252,14 +315,13 @@ class Seams(object):
         :returns:
             A success or fail message if the edges were attached
         '''
-
         body = {
             'parentVertex': parent_id,
             'edgeName': edge_name,
             'edgeVertices': child_vertices
         }
         try:
-            response = self.__http_request(self.bearer, 'POST', '{}/tenant/{}/edge/attach'.format(self.URL, tenant_id), data=body)
+            response = self.__http_request('POST', '{}/tenant/{}/edge/attach'.format(self.URL, tenant_id), json=body)
             return response.text
         except:
             raise SeamsAPIException(response.text)
@@ -287,7 +349,7 @@ class Seams(object):
 
         body = '{{"parentVertexLabel": "{}", "edgeName": "{}", "childVertex": "{}"}}'.format(parent_label, edge_name, child_id)
         try:
-            response = self.__http_request(self.bearer, 'POST', '{}/tenant/{}/edge/attach/label/to'.format(self.URL, tenant_id), data=body)
+            response = self.__http_request('POST', '{}/tenant/{}/edge/attach/label/to'.format(self.URL, tenant_id), data=body)
         except:
             raise SeamsAPIException(response.text)
         return response.text
@@ -315,7 +377,7 @@ class Seams(object):
 
         body = '{{"parentVertex": "{}", "edgeName": "{}", "childVertexLabel": "{}"}}'.format(parent_vertex, edge_name, child_label)
         try:
-            response = self.__http_request(self.bearer, 'POST', '{}/tenant/{}/edge/attach/label/from'.format(self.URL, tenant_id), data=body)
+            response = self.__http_request('POST', '{}/tenant/{}/edge/attach/label/from'.format(self.URL, tenant_id), data=body)
         except:
             raise SeamsAPIException(response.text)
         return response.text
@@ -325,7 +387,7 @@ class Seams(object):
                      tenant_id,
                      caption,
                      *filenames, 
-                     file_type='Files'):
+                     file_type='File'):
         '''
         Bulk upload files
 
@@ -334,7 +396,7 @@ class Seams(object):
         :param *filenames:
             List of filenames the user would like to upload  **REQUIRED**
         :param file_type:
-            Can be 'Files' or 'Images' - defaults to 'Files'
+            Can be 'File' or 'Image' - defaults to 'File'
         :returns:
             A list of vertex id's for the uploaded files
         '''
@@ -345,7 +407,7 @@ class Seams(object):
             files = {'file':(item, open(item, 'rb'))}
 
             try:
-                response = self.__http_request(self.bearer, 'POST', '{}/tenant/{}/upload/file'.format(self.URL, tenant_id), files=files, data=body)
+                response = self.__http_request('POST', '{}/tenant/{}/upload/file'.format(self.URL, tenant_id), files=files, data=body)
                 upload_list.append(response.json())
             except:
                 raise SeamsAPIException('Issue uploading file: {}, response: {}'.format(item, response.text))
@@ -369,7 +431,7 @@ class Seams(object):
         download_list = {}
         for vertex_id in vertex_ids:
             try:
-                response = self.__http_request(self.bearer, 'GET', '{}/tenant/{}/download/file/{}'.format(self.URL, tenant_id, vertex_id))
+                response = self.__http_request('GET', '{}/tenant/{}/download/file/{}'.format(self.URL, tenant_id, vertex_id))
                 download_list[self.__file_name_formatter(response.headers['filename'])] = response.text
             except:
                 raise SeamsAPIException('Issue downloading file: {}, response: {}'.format(vertex_id, response.text))
@@ -394,7 +456,8 @@ class Seams(object):
 
     def __properties_formatter(self, 
                                vertex_label, 
-                               args):
+                               args,
+                               vertex_name=None):
         '''
         Private helper function that formats a list of key value pairs for properties on a vertex
         '''
@@ -402,25 +465,34 @@ class Seams(object):
             if item != 'status':
                 if isinstance(args[item], list):
                     args[item] = json.dumps(args[item])
-        return '{{"vertexLabel": "{}", "properties": {}}}'.format(vertex_label, json.dumps(args))
+        if vertex_name:
+            return '{{"vertexLabel": "{}", "vertexName": "{}", "properties": {}}}'.format(vertex_label, vertex_name, json.dumps(args)).replace("'", "")
+        else:
+            return '{{"vertexLabel": "{}", "properties": {}}}'.format(vertex_label, json.dumps(args)).replace("'", "")
 
 
     def __http_request(self, 
-                       bearer, 
                        req_type, 
                        url, 
                        data=None, 
                        content=None, 
-                       files=None):
+                       files=None,
+                       json=None):
         '''
         Private helper function that makes a specific type of HTTP request based on the req_type
         '''
         
-        header = {'Authorization': 'Bearer {}'.format(bearer)}
+        header = {'Authorization': 'Bearer {}'.format(self.bearer)}
         if content:
             header['Content-Type'] = content
         session = Session()
-        request = Request(req_type, url, headers=header, data=data, files=files)
+        request = Request(req_type, url, headers=header, data=data, files=files, json=json)
         prepared = session.prepare_request(request)
         response = session.send(prepared)
-        return response
\ No newline at end of file
+        return response
+    
+
+    def __chunk_attributes(self, data, SIZE):
+        it = iter(data)
+        for i in range(0, len(data), SIZE):
+            yield {k:data[k] for k in islice(it, SIZE)}
\ No newline at end of file
